@@ -402,10 +402,12 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
          parent->redo();
       return true;
   } else if ( KStandardShortcut::deleteWordBack().contains( key ) ) {
-    parent->deleteWordBack();
+    if (!parent->isReadOnly())
+        parent->deleteWordBack();
     return true;
   } else if ( KStandardShortcut::deleteWordForward().contains( key ) ) {
-    parent->deleteWordForward();
+    if (!parent->isReadOnly())
+       parent->deleteWordForward();
     return true;
   } else if ( KStandardShortcut::backwardWord().contains( key ) ) {
     QTextCursor cursor = parent->textCursor();
@@ -479,6 +481,9 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
   } else if (findReplaceEnabled && KStandardShortcut::findNext().contains(key)) {
       parent->slotFindNext();
       return true;
+  } else if (findReplaceEnabled && KStandardShortcut::findPrev().contains(key)) {
+      parent->slotFindPrevious();
+      return true;
   } else if (findReplaceEnabled && KStandardShortcut::replace().contains(key)) {
       if (!parent->isReadOnly())
           parent->slotReplace();
@@ -558,15 +563,19 @@ QMenu *KTextEdit::mousePopupMenu()
   if (d->findReplaceEnabled) {
       KAction *findAction = KStandardAction::find(this, SLOT(slotFind()), popup);
       KAction *findNextAction = KStandardAction::findNext(this, SLOT(slotFindNext()), popup);
+      KAction *findPrevAction = KStandardAction::findPrev(this, SLOT(slotFindPrevious()), popup);
       if (emptyDocument) {
           findAction->setEnabled(false);
           findNextAction->setEnabled(false);
+          findPrevAction->setEnabled(false);
       } else {
           findNextAction->setEnabled(d->find != 0);
+          findPrevAction->setEnabled(d->find != 0);
       }
       popup->addSeparator();
       popup->addAction(findAction);
       popup->addAction(findNextAction);
+      popup->addAction(findPrevAction);
 
       if (!isReadOnly()) {
           KAction *replaceAction = KStandardAction::replace(this, SLOT(slotReplace()), popup);
@@ -967,6 +976,18 @@ void KTextEdit::slotDoFind()
     slotFindNext();
 }
 
+void KTextEdit::slotFindPrevious()
+{
+    if (!d->find) {
+        return;
+    }
+    const long oldOptions = d->find->options();
+    d->find->setOptions(oldOptions ^ KFind::FindBackwards);
+    slotFindNext();
+    if (d->find) {
+        d->find->setOptions(oldOptions);
+    }
+}
 
 void KTextEdit::slotFindNext()
 {
@@ -1045,6 +1066,7 @@ void KTextEdit::setSpellInterface(KTextEditSpellInterface *spellInterface)
 bool KTextEdit::Private::overrideShortcut(const QKeyEvent* event)
 {
   const int key = event->key() | event->modifiers();
+  qDebug() << key;
 
   if ( KStandardShortcut::copy().contains( key ) ) {
     return true;
@@ -1082,6 +1104,8 @@ bool KTextEdit::Private::overrideShortcut(const QKeyEvent* event)
       return true;
   } else if (findReplaceEnabled && KStandardShortcut::findNext().contains(key)) {
       return true;
+  } else if (findReplaceEnabled && KStandardShortcut::findPrev().contains(key)) {
+      return true;
   } else if (findReplaceEnabled && KStandardShortcut::replace().contains(key)) {
       return true;
   } else if (event->matches(QKeySequence::SelectAll)) { // currently missing in QTextEdit
