@@ -110,7 +110,7 @@ void KRecentDocument::add(const KUrl& url, const QString& desktopEntryName)
     const QString path = recentDocumentDirectory();
     const QString fileName = url.fileName();
     // don't create a file called ".desktop", it will lead to an empty name in kio_recentdocuments
-    const QString dStr = fileName.isEmpty() ? QString("unnamed") : fileName;
+    const QString dStr = path + (fileName.isEmpty() ? QString("unnamed") : fileName);
 
     QString ddesktop = dStr + QLatin1String(".desktop");
 
@@ -119,7 +119,8 @@ void KRecentDocument::add(const KUrl& url, const QString& desktopEntryName)
     while(QFile::exists(ddesktop)){
         // see if it points to the same file and application
         KDesktopFile tmp(ddesktop);
-        if ( tmp.desktopGroup().readEntry("X-KDE-LastOpenedWith") == desktopEntryName ) {
+        if ( tmp.desktopGroup().readPathEntry("URL", QString()) == url.url()
+             && tmp.desktopGroup().readEntry("X-KDE-LastOpenedWith") == desktopEntryName ) {
             KDE::utime(ddesktop, NULL);
             return;
         }
