@@ -116,7 +116,7 @@ class KFileItemDelegate::Private
 
         void gotNewIcon(const QModelIndex& index);
 
-        void paintJobTransfers(QPainter* painter, const qreal& jobAnimationAngle, const QPoint& iconPos, QStyleOptionViewItemV4 opt);
+        void paintJobTransfers(QPainter* painter, const qreal& jobAnimationAngle, const QPoint& iconPos, const QStyleOptionViewItemV4& opt);
 
     public:
         KFileItemDelegate::InformationList informationList;
@@ -307,6 +307,10 @@ QString KFileItemDelegate::Private::information(const QStyleOptionViewItemV4 &op
                     string += item.url().pathOrUrl();
                 break;
 
+            case KFileItemDelegate::Comment:
+                string += item.comment();
+                break;
+
             default:
                 break;
         } // switch (info)
@@ -368,7 +372,7 @@ QSize KFileItemDelegate::Private::layoutText(QTextLayout &layout, const QString
     layout.beginLayout();
     while ((line = layout.createLine()).isValid())
     {
-        line.setLineWidth(int(maxWidth));
+        line.setLineWidth(maxWidth);
         height += leading;
         line.setPosition(QPoint(0, height));
         height += int(line.height());
@@ -475,6 +479,15 @@ QSize KFileItemDelegate::Private::displaySizeHint(const QStyleOptionViewItemV4 &
     setLayoutOptions(layout, option);
 
     QSize size = layoutText(layout, label, maxWidth);
+    if (!info.isEmpty())
+    {
+        // As soon as additional information is shown, it might be necessary that
+        // the label and/or the additional information must get elided. To prevent
+        // an expensive eliding in the scope of displaySizeHint, the maximum
+        // width is reserved instead.
+        size.setWidth(maxWidth);
+    }
+
     return addMargin(size, TextMargin);
 }
 
@@ -853,7 +866,7 @@ void KFileItemDelegate::Private::initStyleOption(QStyleOptionViewItemV4 *option,
     option->showDecorationSelected = true;
 }
 
-void KFileItemDelegate::Private::paintJobTransfers(QPainter *painter, const qreal &jobAnimationAngle, const QPoint &iconPos, QStyleOptionViewItemV4 opt)
+void KFileItemDelegate::Private::paintJobTransfers(QPainter *painter, const qreal &jobAnimationAngle, const QPoint &iconPos, const QStyleOptionViewItemV4& opt)
 {
     painter->save();
     QSize iconSize = opt.icon.actualSize(opt.decorationSize);
@@ -1634,13 +1647,15 @@ bool KFileItemDelegate::eventFilter(QObject *object, QEvent *event)
             return true;
 
         case Qt::Key_Enter:
-        case Qt::Key_Return:
-            if (editor->toPlainText().isEmpty())
+        case Qt::Key_Return: {
+            const QString text = editor->toPlainText();
+            if (text.isEmpty() || (text == QLatin1String(".")) || (text == QLatin1String("..")))
                 return true; // So a newline doesn't get inserted
 
             emit commitData(editor);
             emit closeEditor(editor, SubmitModelCache);
             return true;
+        }
 
         case Qt::Key_Escape:
             emit closeEditor(editor, RevertModelCache);
@@ -1653,9 +1668,15 @@ bool KFileItemDelegate::eventFilter(QObject *object, QEvent *event)
 
     case QEvent::FocusOut:
     {
-        emit commitData(editor);
-        emit closeEditor(editor, NoHint);
-        return true;
+        const QWidget *w = QApplication::activePopupWidget();
+        if (!w || w->parent() != editor)
+        {
+            emit commitData(editor);
+            emit closeEditor(editor, NoHint);
+            return true;
+        }
+        else
+            return false;
     }
 
     default:
